#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <tchar.h>

#define AUDACIOUS_WND_CLASS_NAME    _T("AUDACIOUS_WND_CLASS_NAME")
#define AUDACIOUS_MAGIC_CODE        0x12345678

static HANDLE hMutex;

static LRESULT APIENTRY
WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    COPYDATASTRUCT *lpcds;

    switch (message) {
    case WM_COPYDATA:
        lpcds = (COPYDATASTRUCT *)lParam;

        /* Small security system... */
        if (lpcds->dwData != AUDACIOUS_MAGIC_CODE)
            return 0;

        /* Print command line of other instance */
        MessageBoxA(hWnd, (LPSTR)(lpcds->lpData), "Received from another instance", MB_OK);
        return 0;

    case WM_DESTROY:
        /* Clean up and destroy handler */
        PostQuitMessage(0);
        return 0L;
    }

    return DefWindowProc(hWnd, message, wParam, lParam);
}

static HWND InitApp(HINSTANCE hInstance, int nShowCmd)
{
    TCHAR    szClass[] = AUDACIOUS_WND_CLASS_NAME;
    TCHAR    szTitle[] = _T("");
    HWND     hWnd;
    WNDCLASS wc;

    /* If the class doesn't exists, we must create it */
    if (!GetClassInfo(hInstance, szClass, &wc)) {
        wc.style         = 0;
        wc.lpfnWndProc   = WindowProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = hInstance;
        wc.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
        wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = szClass;
        RegisterClass(&wc);
    }

    /* Create the main window */
    hWnd = CreateWindow(szClass,
                        szTitle,
                        WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT,
                        CW_USEDEFAULT,
                        CW_USEDEFAULT,
                        CW_USEDEFAULT,
                        NULL,
                        NULL,
                        hInstance,
                        NULL);
    if (!hWnd)
        return FALSE;

    ShowWindow(hWnd, nShowCmd);
    UpdateWindow(hWnd);

    return hWnd;
}

BOOL IsAppRunning(LPSTR lpCommandLine)
{
    DWORD          dwRes;
    HWND           hWnd;
    COPYDATASTRUCT cds;

    /* Create the global mutex */
    hMutex = CreateMutex(NULL, TRUE, AUDACIOUS_WND_CLASS_NAME);
    if (GetLastError() != ERROR_ALREADY_EXISTS)
        return FALSE;

    /* Wait that primary instance is initialized */
    dwRes = WaitForSingleObject(hMutex, 1000);

    /* Check if this secondary instance is valid */
    switch (dwRes) {
    case WAIT_OBJECT_0:
    case WAIT_ABANDONED:
        hWnd = FindWindow(AUDACIOUS_WND_CLASS_NAME, NULL);

        if (hWnd)
        {
            /* Put the window on top of Z order */
            SetForegroundWindow(hWnd);

            /* Restore the window if it was minimized */
            if (IsIconic(hWnd))
            {
                ShowWindow(hWnd, SW_RESTORE);
                UpdateWindow(hWnd);
            }

            /* Numeric information (I use it as stupid "security") */
            cds.dwData = AUDACIOUS_MAGIC_CODE;
            /* Size of the transfer buffer */
            cds.cbData = strlen(lpCommandLine) + 1;
            /* Command line to be sent to primary instance. */
            cds.lpData = lpCommandLine;

            SendMessage(hWnd, WM_COPYDATA, 0, (LPARAM)&cds);
        }

        break;
    }

    /* Close duplicated handle */
    CloseHandle(hMutex);

    return TRUE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
    MSG msg;

    if (IsAppRunning(lpCmdLine))
    {
        return 0;
    }

    if (InitApp(hInstance, nShowCmd) == NULL) {
        return 1;
    }

    ReleaseMutex(hMutex);

    for (;;)
    {
        if (GetMessage(&msg, NULL, 0, 0) <= 0)
            break;

        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    CloseHandle(hMutex);

    return 0;
}
